/*
 * Copyright (c) 2005 The University of Utah
 *
 * This file is part of Jiazzi.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions, and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package out;

/**
 * @author eeide
 *
 */
public class Num {
    final int value;

    public static Num maybeNum(String s) {
        try {
            int n = Integer.parseInt(s);
            return new Num(n);
        } catch (NumberFormatException nfe) {
            // Nothing to do here.
        }
        return null;
    }

    public Num(Num n) {
        super();
        value = n.value;
    }

    protected Num(int i) {
        super();
        value = i;
    }

    public int getValue() {
        return value;
    }

    public String toString() {
        return Integer.toString(value);
    }

    /*****/

    public boolean isZero() {
        return (value == 0);
    }

    // Return a new Num, or null to indicate failure.
    public Num makeNegate() {
        return new Num(-value);
    }

    public Num makeSum(Num n) {
        return new Num(value + n.value);
    }

    public Num makeDifference(Num n) {
        return new Num(value - n.value);
    }

    public Num makeProduct(Num n) {
        return new Num(value * n.value);
    }

    public Num makeRatio(Num n) {
        if (n.isZero()) {
            return null;
        }
        return new Num(value / n.value);
    }
}
