/*
 * Copyright (c) 2005 The University of Utah
 *
 * This file is part of Jiazzi.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions, and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package out;

import java.io.InputStream;
import java.io.PrintStream;

import expr.Evaluator;
import expr.Expr;
import expr.Printer;
import parse.Parser;
import input.Input;

/**
 * @author eeide
 *
 */
public class REPL {
    protected final Input input;
    protected final Parser parser;
    protected final Evaluator evaluator;
    protected final Printer printer;
    protected PrintStream output;

    public REPL(InputStream in, PrintStream out) {
        super();
        input = new Input(in);
        parser = new Parser();
        evaluator = new Evaluator();
        printer = new Printer(out);
        this.output = out;
    }

    public boolean repl1() {
        String s = input.getInput();
        if (s == null) {
            return false;
        }
        Expr e = parser.parse(s);
        printer.print(e);
        output.println();
        Expr result = evaluator.eval(e);
        printer.print(result);
        output.println();
        return true;
    }

    public void repl() {
        for (;;) {
            if (!repl1()) {
                break;
            }
        }
    }
}
